#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "ansi.h"
#include "common_errors.h"
#include "parse_info.h"
#include "read_line.h"

// Copia saltando skip caratteri e i doppi apici
char * copy_token( char *source, int skip ) {
  char *target;
  int len;
  target = NULL;
  len = strlen(source) - skip;
  target = (char*) calloc( len+1, sizeof(char) ); 
  strncpy( target, source + skip, len );
  while ( target[0] == '"' ) {
    target++;
    len--;
  }
  len--;
  while ( target[len] == '"' ) {
    target[len]=0;
    len--;
  }
  return target;
}



// Decodifica il file INFO con le informazioni sul pacchetto
int parse_info(char *infofile, TPackageInfo *lpstInfo ){
  char line_buffer[256];
  FILE *fd;

  char *tempstr;
  int  descr_flag;
  int l;

  // Pulisce la struttura prima di inziare
  bzero(lpstInfo, sizeof(TPackageInfo) );

  // Apre il file INFO    
  fd = fopen(infofile, "r");
  if (fd == NULL) {
    printERROR("Impossibile aprire il file %s, probabilmente non e' un pacchetto BP !\n", infofile );
    normal_color();
    return ERRORE_FORMATO;
  }
  
  // Parsing
  line_buffer[0]=0;
  descr_flag=0;
  while ( ! feof(fd) ) {
    if ( 0 == read_line(fd,line_buffer)) continue;
    // parse
    if ( 0 == strncmp( TSONAME, line_buffer, LSONAME ) ) {
      lpstInfo->soname = copy_token(line_buffer, LSONAME );
      descr_flag = 0;
    } else
    if ( 0 == strncmp( TVERSION, line_buffer, LVERSION ) ) {
      lpstInfo->version = copy_token(line_buffer, LVERSION );
      descr_flag = 0;
    } else
    if ( 0 == strncmp( TCATEGORY, line_buffer, LCATEGORY ) ) {
      lpstInfo->category = copy_token(line_buffer, LCATEGORY );
      descr_flag = 0;
    } else
    if ( 0 == strncmp( TDISKNAME,line_buffer, LDISKNAME ) ) {
      lpstInfo->diskname = copy_token(line_buffer, LDISKNAME );
      descr_flag = 0;
    } else
    if ( 0 == strncmp( TARCH, line_buffer, LARCH ) ) {
      lpstInfo->arch = copy_token(line_buffer, LARCH );
      descr_flag = 0;
    } else
    if ( 0 == strncmp( TOPTIM, line_buffer, LOPTIM ) ) {
      lpstInfo->optim = copy_token(line_buffer, LOPTIM );
      descr_flag = 0;
    } else
    if ( 0 == strncmp( TSUMMARY, line_buffer, LSUMMARY ) ) {
      lpstInfo->summary = copy_token(line_buffer, LSUMMARY );
      descr_flag = 0;
    } else
    if ( 0 == strncmp( TDISKUSAGE, line_buffer, LDISKUSAGE ) ) {
      tempstr = copy_token(line_buffer, LDISKUSAGE );
      if (tempstr==NULL) {
        lpstInfo->diskusage = 0;
      } else {
        lpstInfo->diskusage = atol(tempstr);
      }
      descr_flag = 0;
    } else
    if ( 0 == strncmp( TSTATUS, line_buffer, LSTATUS ) ) {
      lpstInfo->status = copy_token(line_buffer, LSTATUS );
      descr_flag = 0;
    } else
    if ( 0 == strncmp( TAUTHOR, line_buffer, LAUTHOR ) ) {
      lpstInfo->author = copy_token(line_buffer, LAUTHOR );
      descr_flag = 0;
    } else
    if ( 0 == strncmp( TCOPYRIGHT, line_buffer, LCOPYRIGHT ) ) {
      lpstInfo->copyright = copy_token(line_buffer, LCOPYRIGHT );
      descr_flag = 0;
    } else
    if ( 0 == strncmp( TMANTAINER, line_buffer, LMANTAINER ) ) {
      lpstInfo->mantainer = copy_token(line_buffer, LMANTAINER );
      descr_flag = 0;
    } else
    if ( 0 == strncmp( TURL, line_buffer, LURL ) ) {
      lpstInfo->url = copy_token(line_buffer, LURL );
      descr_flag = 0;
    } else
    if ( 0 == strncmp( TFTPURL, line_buffer, LFTPURL ) ) {
      lpstInfo->ftpurl = copy_token(line_buffer, LFTPURL );
      descr_flag = 0;
    } else
    if ( 0 == strncmp( TOS, line_buffer, LOS ) ) {
      lpstInfo->os = copy_token(line_buffer, LOS );
      descr_flag = 0;
    } else
    if ( 0 == strncmp( TDISTRIBUTION, line_buffer, LDISTRIBUTION ) ) {
      lpstInfo->distribution = copy_token(line_buffer, LDISTRIBUTION );
      descr_flag = 0;
    } else
    if ( 0 == strncmp( TBUILDHOST, line_buffer, LBUILDHOST ) ) {
      lpstInfo->buildhost = copy_token(line_buffer, LBUILDHOST );
      descr_flag = 0;
    } else
    if ( 0 == strncmp( TBUILDTIME, line_buffer, LBUILDTIME ) ) {
      lpstInfo->buildtime = copy_token(line_buffer, LBUILDTIME );
      descr_flag = 0;
    } else
    if ( 0 == strncmp( TBPPVERSION, line_buffer, LBPPVERSION ) ) {
      lpstInfo->bppversion = copy_token(line_buffer, LBPPVERSION );
      descr_flag = 0;
    } else
    // Get Description
    if ( 0 == strncmp( TDESCRIPTION,line_buffer, LDESCRIPTION ) ) {
      lpstInfo->description = copy_token(line_buffer, LDESCRIPTION );
      descr_flag = 1;
      //printf("descr:%s:\n", lpstInfo->description );
    } else
    if ( descr_flag == 1 ) {
      l = strlen(lpstInfo->description) + strlen(line_buffer) + 3;
      tempstr=strdup(lpstInfo->description);
      lpstInfo->description = (char*) malloc(l);
      strcpy( lpstInfo->description, tempstr );
      strcat( lpstInfo->description, " \n" );
      free(tempstr);
      while(line_buffer[strlen(line_buffer)-1] == '"' )line_buffer[strlen(line_buffer)-1]=0;
      strcat( lpstInfo->description,line_buffer );
      //printf("descr2:%s:\n", lpstInfo->description );
    } else
 {
      printWARNING("Parametro ignorato: %s \n",line_buffer );
      normal_color();
    }
    // Next Line  
  }
  fclose(fd);
  //printf("- Parsing completed !\n");
  return 0;
}


// Self explain :-)
void debug_info( TPackageInfo *lpstInfo ) {
  printf("[ Debug Package Info ]\n");
  printf("  Soname............: %s\n", lpstInfo->soname );
  printf("  Version ..........: %s\n", lpstInfo->version );
  printf("  Category .........: %s\n", lpstInfo->category );
  printf("  Disk Name ........: %s\n", lpstInfo->diskname );
  printf("  Architettura min. : %s\n", lpstInfo->arch );
  printf("  Architettura opt. : %s\n", lpstInfo->optim );
  printf("  Summary ......... : %s\n", lpstInfo->summary );
  printf("  Disk Usage ...... : %li\n", lpstInfo->diskusage );
  printf("  Status .......... : %s\n", lpstInfo->status );
  printf("  Author .......... : %s\n", lpstInfo->author );
  printf("  Copyright ....... : %s\n", lpstInfo->copyright );
  printf("  Description lenght: %i\n", strlen(lpstInfo->description) );
  printf("  Mantainer ....... : %s\n", lpstInfo->mantainer );
  printf("  URL ............. : %s\n", lpstInfo->url );
  printf("  FTP URL ......... : %s\n", lpstInfo->ftpurl );
  printf("  OS .............. : %s\n", lpstInfo->os );
  printf("  Distribution .... : %s\n", lpstInfo->distribution );
  printf("  Build Host .......: %s\n", lpstInfo->buildhost );
  printf("  Build Time ...... : %s\n", lpstInfo->buildtime );
  printf("  BPP Version ..... : %s\n", lpstInfo->bppversion );
  printf("<Description>%s</Description>\n", lpstInfo->description );
}
