#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>

#include "ansi.h"
#include "check_package_extension.h"
#include "common_errors.h"
#include "concatenate.h"
#include "dirbase.h"
#include "dirpath.h"
#include "file_exists.h"
#include "parse_options.h"
#include "setup_help.h"
#include "setup_main.h"



// Analizza e ricava i parametri della riga di comando di setup
// riempiendo la struttura delle opzioni
int parse_options( int *lpi, char ***lpac, TSetupOptions *lpopt ) {
  int i;
  //printf("- analisi di %i parametri per %s \n", *lpi, (*lpac)[0] );

  // Se non ci sono sufficienti parametri richiama la guida in linea
  if ( *lpi < 2 ) {
    printERROR("Numero insufficiente di parametri.\n" );
    normal_color();
    setup_help();
  }
  // Analisi parametri
  bzero( lpopt, sizeof(TSetupOptions) );
  for ( i=1; i<*lpi; i++ ) {
    //printf("- analisi parametro nr. %i : %s \n", i, (*lpac)[i] );
    if ( 0 == strcmp( "-i", (*lpac)[i]) )  {
      lpopt->action = ACTION_INSTALL;
      lpopt->interactive=0;
    } else
    if ( 0 == strcmp( "--install", (*lpac)[i]) )  {
      lpopt->action = ACTION_INSTALL;
      lpopt->interactive=0;
    } else
    if ( 0 == strcmp( "-y", (*lpac)[i]) ) 
      lpopt->action = ACTION_FINALIZE;
    else
    if ( 0 == strcmp( "--finalize", (*lpac)[i]) ) 
      lpopt->action = ACTION_FINALIZE;
    else
    if ( 0 == strcmp( "-z", (*lpac)[i]) ) 
      lpopt->action = ACTION_POSTINSTALL;
    else
    if ( 0 == strcmp( "--post-install", (*lpac)[i]) ) 
      lpopt->action = ACTION_POSTINSTALL;
    else
    if ( 0 == strcmp( "-x", (*lpac)[i]) ) 
      lpopt->action = ACTION_XPOSTINSTALL;
    else
    if ( 0 == strcmp( "--x-post-install", (*lpac)[i]) ) 
      lpopt->action = ACTION_XPOSTINSTALL;
    else
    if ( 0 == strcmp( "-u", (*lpac)[i]) ) 
      lpopt->action = ACTION_UPDATE;
    else
    if ( 0 == strcmp( "--update", (*lpac)[i]) ) 
      lpopt->action = ACTION_UPDATE;
    else
    if ( 0 == strcmp( "-k", (*lpac)[i]) ) 
      lpopt->action = ACTION_RESTORE;
    else
    if ( 0 == strcmp( "--restore", (*lpac)[i]) ) 
      lpopt->action = ACTION_RESTORE;
    else
    if ( 0 == strcmp( "-d", (*lpac)[i]) ) 
      lpopt->action = ACTION_DELETE;
    else
    if ( 0 == strcmp( "--delete", (*lpac)[i]) ) 
      lpopt->action = ACTION_DELETE;
    else
    if ( 0 == strcmp( "-p", (*lpac)[i]) ) 
      lpopt->action = ACTION_PURGE;
    else
    if ( 0 == strcmp( "--purge", (*lpac)[i]) ) 
      lpopt->action = ACTION_PURGE;
    else
    if ( 0 == strcmp( "-c", (*lpac)[i]) ) 
      lpopt->action = ACTION_CHECK;
    else
    if ( 0 == strcmp( "--check", (*lpac)[i]) ) 
      lpopt->action = ACTION_CHECK;
    else
    if ( 0 == strcmp( "--md5check", (*lpac)[i]) ) 
      lpopt->action = ACTION_MD5CHECK;
    else
    if ( 0 == strcmp( "--h", (*lpac)[i]) ) 
      setup_help();
    else
    if ( 0 == strcmp( "--help", (*lpac)[i]) ) 
      setup_help();
    /* Opzioni */
    else
    if ( 0 == strcmp( "--no-header", (*lpac)[i]) ) 
      lpopt->noheader=1;
    else
    if ( 0 == strcmp( "-f", (*lpac)[i]) ) {
      lpopt->force=1; 
      lpopt->relaxed=0; 
    } else
    if ( 0 == strcmp( "--force", (*lpac)[i]) ) { 
      lpopt->force=1; 
      lpopt->relaxed=0; 
    } else
    if ( 0 == strcmp( "--relaxed", (*lpac)[i]) ) {
      lpopt->force=0; 
      lpopt->relaxed=1; 
    } else
    if ( 0 == strcmp( "-v", (*lpac)[i]) ) 
      lpopt->verbose++;
    else
    if ( 0 == strcmp( "--verbose", (*lpac)[i]) ) 
      lpopt->verbose++;
    else 
    if ( 0 == strcmp( "--debug", (*lpac)[i]) ) 
      lpopt->debug=1;
    else 
    /* Default: probabilmente un parametro esteso */
    {
      // Controlla se e' un 'installazione interattiva
      if (( (*lpac)[i][0] != '-' ) && ( lpopt->sourcefile == NULL )) {
        lpopt->sourcefile = strdup( (*lpac)[i] );
        //printf("- pacchetto da installare: %s \n", lpopt->sourcefile );	
        if ( lpopt->action == ACTION_UNKNOWN ) {
          //printf("- seleziono installazione interattiva \n" );	
          lpopt->action = ACTION_INSTALL;
          lpopt->targetdir = strdup("/");
	  lpopt->interactive=1;
	} else {
          printWARNING("- Seleziono l'installazione non interattiva \n" );
	  normal_color();
	  lpopt->interactive=0;
	}
      } else
      // Se il nome del pacchetto e' stato gia inserito si tratta di una dir
      if (( (*lpac)[i][0] != '-' ) && ( lpopt->targetdir == NULL )) {
        lpopt->targetdir = strdup( (*lpac)[i] );
        //printf("- directory d'installazione: %s \n", lpopt->targetdir );		
      } else {
        printERROR("Il parametro (%i) [%s] non e' stato riconosciuto.\n", i, (*lpac)[i] );
	normal_color();
        exit ( ERRORE_PARAMETRO );
      }
      // end of default
    }
  // next i
  }  
  // Ok
  return 0;
}



// Controlla i dati nella struttura delle opzioni a seconda dell'azione
int check_options( TSetupOptions *lpopt ) {
  switch( lpopt->action ) {
  
    // Almeno 1 action deve essre stata scelta !
    case ACTION_UNKNOWN: {
      printERROR( "Non e' stata specificata l'azione da eseguire.\n");
      normal_color();
      exit ( ERRORE_PARAMETRO );
    }  

    case ACTION_INSTALL:
    case ACTION_UPDATE:
    case ACTION_RESTORE: {
      // Il pacchetto deve essere specificato
      if ( lpopt->sourcefile == NULL ) {
        printERROR( "Non e' stata specificato il nome del pacchetto.\n");
	normal_color();
        exit ( ERRORE_PARAMETRO );
      }  
      // If package does not exist try appending a name to it
      if ( 0 != file_exists(lpopt->sourcefile) ) {
        if ( 0 == check_package_extension( &(lpopt->sourcefile), ".bpp") ) {
          printWARNING( "- Utilizzo %s come nome pacchetto.\n", lpopt->sourcefile );
	  normal_color();
	} else
	if ( 0 == check_package_extension( &(lpopt->sourcefile), ".tgz") ) {
          printWARNING( "- Utilizzo %s come nome pacchetto.\n", lpopt->sourcefile );
	  normal_color();
	} else {
	  printERROR( "Pacchetto non trovato %s\n", lpopt->sourcefile );
	  normal_color();
	  exit( ERRORE_NOTFOUND );
	}
      }
      // La directory deve essere specificata               
      if ( lpopt->targetdir == NULL ) {
        printWARNING( "- Non e' stata specificata la directory di destinazione, presumo /.\n");
	normal_color();
        lpopt->targetdir = strdup("/");
	lpopt->interactive = 1;
      }  
      // La directory deve esistere
      if ( 0 != file_exists(lpopt->targetdir) ) {
        printERROR( "Directory d'installazione %s inesistente.\n", lpopt->targetdir );
	normal_color();
        exit( ERRORE_NOTFOUND );
      }
      break;
    }

    case ACTION_FINALIZE: {
      // Directory di default
      if ( lpopt->sourcefile == NULL ) {
	printWARNING( "- Non e' stata specificata la directory della distribuzione, presumo /cdrom\n");
	normal_color();
        lpopt->sourcefile = strdup("/cdrom");
	lpopt->targetdir = strdup("/");
      }
      if ( lpopt->sourcedir == NULL ) {
        lpopt->sourcedir = strdup( lpopt->sourcefile );
      }
      // La directory (come filename) deve esistere
      if ( 0 != file_exists(lpopt->sourcefile) ) {
        printERROR( "Directory d'installazione %s inesistente.\n", lpopt->sourcefile );
	normal_color();
        exit( ERRORE_NOTFOUND );
      }
      // Directory di default
      if ( lpopt->targetdir == NULL ) {
        printWARNING( "- Non e' stata specificata la directory di destinazione, presumo /.\n");
	normal_color();
        lpopt->targetdir = strdup("/");
      }  
      // La directory deve esistere
      if ( 0 != file_exists(lpopt->targetdir) ) {
        printERROR( "Directory d'installazione %s inesistente.\n", lpopt->targetdir );
	normal_color();
        exit( ERRORE_NOTFOUND );
      }
      break;
    }  

    case ACTION_POSTINSTALL: 
    case ACTION_XPOSTINSTALL: {
      // Ignora parametro nomefile
      if ( lpopt->sourcefile != NULL ) {
        printWARNING( "Il parametro %s e' stato ignorato.\n", lpopt->sourcefile );
        normal_color();
      }  
      // Ignora parametro directory
      if ( lpopt->targetdir != NULL ) {
        printWARNING( "Il parametro %s e' stato ignorato.\n", lpopt->targetdir );
	normal_color();
      }
      // queste action possono essere SOLO locali
      lpopt->sourcefile = strdup("/");
      lpopt->targetdir  = strdup("/");
      lpopt->sourcedir  = strdup("/");
      break;
    }

    case ACTION_DELETE: 
    case ACTION_CHECK: {
      // Il pacchetto deve essere specificato
      if ( lpopt->sourcefile == NULL ) {
        printERROR( "Non e' stata specificato il nome del pacchetto.\n");
	normal_color();
        exit ( ERRORE_PARAMETRO );
      }  
      // La directory deve essere specificata               
      if ( lpopt->targetdir == NULL ) {
        printWARNING( "- Non e' stata specificata la directory di destinazione, presumo /.\n");
	normal_color();
        lpopt->targetdir = strdup("/");
      }  
      // La directory deve esistere
      if ( 0 != file_exists(lpopt->targetdir) ) {
        printERROR( "Directory d'installazione %s inesistente.\n", lpopt->targetdir );
	normal_color();
        exit( ERRORE_NOTFOUND );
      }
      break;
    }
    // Il metodo scelto non e' ancora gestito
    default: {
      printERROR( "Metodo d'installazione %i non ancoa gestito.\n", lpopt->action );
      normal_color();
      exit( ERRORE_PARAMETRO );  
    }
  // End of switch  
  }
  // ok
  return 0;
}



// Calcola alcuni valori non specificati
int adjust_options( TSetupOptions *lpopt ) {
  int i;
  char *s;
  
  // Calcolo della Sourcedir
  if ( lpopt->sourcedir == NULL ) {
    lpopt->sourcedir = dirpath( lpopt->sourcefile );
    if ( 0 == strcmp( ".", lpopt->sourcedir ) ) {
      lpopt->sourcedir = concatenate( getcwd( NULL, 0 ), "/" );
      lpopt->sourcefile = concatenate( lpopt->sourcedir, lpopt->sourcefile );
    }
  }

  // Calcolo nome completo del pacchetto ('dirbase $i .bpp')
  lpopt->fullname = dirbase(lpopt->sourcefile);  
  i = strlen(lpopt->fullname);
  if ( 0 == strcmp( lpopt->fullname+i-4, ".bpp" ) ) {
    lpopt->fullname[i-4] = 0;
    lpopt->sourceext = strdup(".bpp");
  } else
  if ( 0 == strcmp( lpopt->fullname+i-4, ".tgz" ) ) {
    lpopt->fullname[i-4] = 0;
    lpopt->sourceext = strdup(".tgz");
  }

  // Calcolo SoName e Version
  s = strrchr( lpopt->fullname, '_' );
  if ( s == NULL ) {
    lpopt->soname  = strdup(lpopt->fullname);
    lpopt->version = strdup("");
  } else {
    i = strlen(lpopt->fullname);
    while ( (i>0) && (lpopt->fullname[i] != '_') ) i--;
    lpopt->soname  = malloc( sizeof(char) * (i + 1) );
    strncpy( lpopt->soname, lpopt->fullname, i );
    lpopt->soname[i] = 0;
    lpopt->version = malloc( sizeof(char) * ( strlen(lpopt->fullname) - i + 1) );
    lpopt->version[0] = 0;
    strcat( lpopt->version, lpopt->fullname+i+1 );
    
  }
  
  // Calcolo directory di spool
  lpopt->spoolroot = concatenate( lpopt->targetdir, SETUP_SPOOLROOT );
  
  // Calcolo directory pacchetto
  lpopt->spooldir = concatenate( lpopt->spoolroot, lpopt->fullname );
  return 0;
}



// Stampa il debug delle opzioni
int debug_options( TSetupOptions *lpopt ) {
  if ( lpopt->debug == 0 ) return 0;
  printf( "\n" );
  printf( "[  Debug Header  ] \n");
  printf( "Source File .....: %s\n", lpopt->sourcefile );
  printf( "Source Dir ......: %s\n", lpopt->sourcedir );
  printf( "Source Ext ......: %s\n", lpopt->sourceext );
  printf( "Soname ..........: %s\n", lpopt->soname );
  printf( "Version .........: %s\n", lpopt->version );
  printf( "Full Name .......: %s\n", lpopt->fullname );
  printf( "\n" );
  printf( "Target Dir ......: %s\n", lpopt->targetdir );
  printf( "Spool Root ......: %s\n", lpopt->spoolroot );
  printf( "Spool Dir .......: %s\n", lpopt->spooldir );
  printf( "\n" );
  printf( "Action ..........: %i\n", lpopt->action );
  printf( "Interactive .....: %i\n", lpopt->interactive );
  printf( "No Header .......: %i\n", lpopt->noheader );
  printf( "Force ...........: %i\n", lpopt->force );
  printf( "Relaxed .........: %i\n", lpopt->relaxed );
  printf( "Verbose .........: %i\n", lpopt->verbose );
  sleep(3);
  return 0;
}


// Esce con errore solo se non relaxed
void uscita(int errore, TSetupOptions *lpopt) {
  if (lpopt->relaxed == 1) { 
    exit(0); 
  } else { 
    exit(errore); 
  }
}
