#include <config.h>

#include <stdio.h>
#include <unistd.h>

#include "ansi.h"
#include "common_errors.h"
#include "deltree.h"
#include "file_exists.h"
#include "makedir.h"
#include "parse_options.h"
#include "setup_finalize.h"
#include "setup_main.h"
#include "setup_install.h"
#include "setup_post_install.h"


int main( int argc, char *argv[] ) {
  TSetupOptions lostOptions;

  // Parse Options
  parse_options( &argc, &argv, &lostOptions );

  // Print Version
  if ( lostOptions.noheader == 0 ) {
    printTITLE("AGX's Package Installer - %s_%s \n", PACKAGE, VERSION );
    normal_color();
  }

  // Check parsed options
  check_options( &lostOptions );
  // Adjust some options
  adjust_options( &lostOptions );
  // Print out debug infos
  debug_options( &lostOptions );

  // Clean Up FS
  chdir( lostOptions.targetdir );  
  deltree( "install" );
  deltree( "incoming" );
  makedir( lostOptions.spoolroot, 00711 );

  // Controllo Installazione pacchetto
  switch (lostOptions.action) {
    case ACTION_UPDATE:
    case ACTION_RESTORE:
    case ACTION_DELETE:
    case ACTION_PURGE:
    case ACTION_CHECK: {
      if ( 0 != file_exists(lostOptions.spooldir) ) {
        printERROR("Il pacchetto %s non e' installato in %s\n", lostOptions.fullname, lostOptions.targetdir );
	normal_color();
        uscita(ERRORE_NONINSTALLATO,&lostOptions);
      }
      break;
    }    
  }	

  // Esegue l'azione
  switch (lostOptions.action) {
    case ACTION_INSTALL:
    case ACTION_UPDATE:
    case ACTION_RESTORE: {
      setup_install( &lostOptions );
      break;
    }
    case ACTION_FINALIZE: {
      setup_finalize( &lostOptions );
      break;
    }
    // TODO: complete ... :-)
    case ACTION_POSTINSTALL:
    case ACTION_XPOSTINSTALL: {
      setup_post_install( &lostOptions );
      break;
    }
    /*
  REMOVE|PURGE)
    exec setup--remove
  CHECK|MD5CHECK)
    exec setup--check
  UPGRADE)
    exec setup--upgrade
esac
*/
    default: {
      printERROR( "La modalita' di funzionamento %i non e' supportata.\n", lostOptions.action );
      normal_color();
      exit(ERRORE_PARAMETRO);    
    }
  }
  return 0;
}
