#include <sys/stat.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include "blockcache.h"
#include "common_errors.h"
#include "file_time.h"
#include "gnutar.h"
#include "namecache.h"
#include "support.h"

extern time_t      gstime;

// Handle a cache of block containing information on the owner
// and permission to set on the file system, permissions are
// handled from bottom to top
TCacheBlock *firstBlock = NULL;
TCacheBlock *lastBlock  = NULL;


// Insert a new block into the double linked list of blocks
int push_block( char *target_file, TTarBlock *buffer ) {
  TCacheBlock *newBlock;
  newBlock = (TCacheBlock*) calloc( 1, sizeof(TCacheBlock) );
  if ( firstBlock == NULL ) {
    firstBlock = newBlock;
    lastBlock  = newBlock;
    //printf("push_block: aggiunto primo blocco %s\n", target_file);
  } else {
    lastBlock->next = newBlock;
    newBlock->prev = lastBlock;
    lastBlock = newBlock;
    //printf("push_block: aggiunto blocco %s\n", target_file);
  }
  // Decode values
  newBlock->name = (char*) strdup(target_file);
  newBlock->mode = oct2int( buffer->mode, 8 );
  newBlock->uid = lookup_user( buffer->uname );
  newBlock->gid = lookup_group( buffer->gname );
  newBlock->mtime = oct2int( buffer->mtime, 12 );
  return 0;
}


// Flush the block cache applying permissions to the FS from bottom to top
int FlushCache() {
  int retval = 0;
  if ( lastBlock == NULL ) {
    printf("flushcache: No block to flush !\n");
    return 0;
  }
  //printf("- Flushing block cache \n");
  do {
    // Apply values
    // set the utime before chmod because some versions of utime are
    // broken and trash the modes of the file.
    //printf("- untar_FlushCache: %s\n", lastBlock->name );    
    if ( 0 != file_time( lastBlock->name, lastBlock->mtime, gstime ) ) {
      printf("untar: errore in file_time %s\n", lastBlock->name );
      retval = ERRORE_IO;
    }
    if ( 0 != chown( lastBlock->name, lastBlock->uid, lastBlock->gid ) ) {
      printf("untar: errore in chown %s\n", lastBlock->name );
      retval = ERRORE_IO;
    }
    if ( 0 != chmod( lastBlock->name, lastBlock->mode ) ) {
      printf("untar: errore in chmod %s\n", lastBlock->name );
	retval = ERRORE_IO;
    }
    // Go previous and free
    lastBlock = lastBlock->prev;
    if (lastBlock != NULL ) { 
      free(lastBlock->next);
      lastBlock->next = NULL;
    }
  } while ( lastBlock != NULL );
  free(firstBlock);
  firstBlock= NULL;
  return retval;
}
