#ifndef _GNUTAR_H
#define _GNUTAR_H

/* 
   A tar archive consists of 512-byte blocks.
   Each file in the archive has a header block followed by 0 or more
   data blocks.
   Two blocks of NUL bytes indicate the end of the archive.  
*/

#define TAR_BLOCKSIZE 512

/* 
   The fields of the header blocks are all numeric if not specified.
   Numbers are ISO 646 representations of octal numbers, with leading 
   zeros as needed. Strings stored as ISO 646 (approximately ASCII) strings.
*/

typedef struct tar_block TTarBlock;

struct tar_block {		/* Offset */
  char name[100];		/*   0, File Name NULL Terminated */
  char mode[8];			/* 100, File permissions */
  char uid[8];			/* 108, user ID */
  char gid[8];			/* 116, group ID */
  char size[12];		/* 124, File Size */
    /* If typeflag=={LNKTYPE,SYMTYPE,DIRTYPE} then size must be 0. */
  char mtime[12];		/* 136, Modification time */
  char chksum[8];		/* 148 */
    /* chksum contains the sum of all 512 bytes in the header block,
       treating each byte as an 8-bit unsigned value and treating the
       8 bytes of chksum as blank characters. */
  char typeflag;		/* 156, Type of file (see below) */
  char linkname[100];		/* 157, NULL Terminated */
    /* linkname is only valid when typeflag==LNKTYPE.  It doesn't use prefix;
       files that are links to pathnames >100 chars long can not be stored
       in a tar archive. */
  char magic[6];		/* 257, TMAGIC NULL Terminated */
  char version[2];		/* 263, TVERSION */
  char uname[32];		/* 265, NULL Terminated */
  char gname[32];		/* 297, NULL Terminated */
    /* uname and gname are used in preference to uid and gid, if those
       names exist locally. */
  char devmajor[8];		/* 329 */
  char devminor[8];		/* 337 */
    /* devmajor and devminor are only valid for typeflag=={BLKTYPE,CHRTYPE}. */
  char prefix[155];		/* 345, NULL Terminated */
    /* If the first character of prefix is '\0', the file name is name;
       otherwise, it is prefix/name.  Files whose pathnames don't fit in that
       length can not be stored in a tar archive.  */
  char reserved[11];		/* 500, Fill in space, all \0 */
};


/* Tar 'mode' field: */
#define TAR_SUID	04000
#define TAR_SGID	02000
#define TAR_SVTX	01000
#define TAR_UREAD	00400
#define TAR_UWRITE	00200
#define TAR_UEXEC	00100
#define TAR_GREAD	00040
#define TAR_GWRITE	00020
#define TAR_GEXEC	00010
#define TAR_OREAD	00004
#define TAR_OWRITE	00002
#define TAR_OEXEC	00001

/* Tar 'type' field: */
#define TAR_REGTYPE	'0'	/* Regular file (preferred code).  */
#define TAR_AREGTYPE	'\0'	/* Regular file (alternate code).  */
#define TAR_LNKTYPE	'1'	/* Hard link.  */
#define TAR_SYMTYPE	'2'	/* Symbolic link (hard if not supported).  */
#define TAR_CHRTYPE	'3'	/* Character special.  */
#define TAR_BLKTYPE	'4'	/* Block special.  */
#define TAR_DIRTYPE	'5'	/* Directory.  */
#define TAR_FIFOTYPE	'6'	/* FIFO, i.e. Named pipe.  */
#define TAR_CONTTYPE	'7'	/* Contiguous file */

/* Tar 'magic' field and its length.  */
#define TAR_MAGIC	"ustar"
#define TAR_MAGLEN	6

/* Tar 'version' field and its length.  */
#define TAR_VERSION	"00"
#define TAR_VERSLEN	2

#endif
