#include <grp.h>
#include <pwd.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "namecache.h"

// This is a cache of Name and Group permissions
// Permissions are stored in tar into text and numeric format,
// because the numeric (uid & gid) can change between two systems,
// depending on the passwd and group files, we lookup for the value
// of the selected user or group into the corrispondending file
// we cache the value into thos single list to avoid going on the FS
// every time we look-up for a name. If the user exist we use the value
// from the passwd file, otherwise we use the uid stored into the
// tar archive. This can be broken ?
TCachedName *firstCachedUser  = NULL;
TCachedName *firstCachedGroup = NULL;

int lookup_user( char *s ){
  TCachedName *iname = NULL;
	// Counter
  TCachedName *jname = NULL;	// Point to the tail
  TCachedName *lname = NULL;	// Current
  struct passwd *lostpwd = NULL;
  int retval = 0;
  
  // First, look-up the name into the cache    
  iname = firstCachedUser;
  jname = firstCachedUser;
  lname = NULL;
  while ( iname != NULL ) {
    //printf("lookup: %s\n", iname->name );
    if ( 0 == strcmp(s, iname->name) ) break;
    jname = iname;
    iname = iname->next;
  }
  // If not found go into the passwd file, adding a new entry
  // to the user name cache
  if (iname == NULL ) {
    //printf("scanning for %s\n", s );
    lname = (TCachedName*) calloc( 1, sizeof(TCachedName));
    lname->name = (char*) strdup(s);
    lname->uid = 0;
    lname->gid = 0;
    lname->next = NULL;
    if (firstCachedUser==NULL) {
      firstCachedUser = lname;
	// Add as head
    } else {
      jname->next = lname;
	// Append to the tail
    }
    // Now find the user in passwd
    lostpwd = getpwnam(s);
    lname->uid = lostpwd->pw_uid;  
    retval = lname->uid;
    //printf("- namecache: %s == %i \n", s, retval );
  } else {
    retval = iname->uid;
    //printf("- namecache: found %s at %s \n", s, iname->name );
  }
  return retval;
}


// Same as look-up user
int lookup_group( char *s ){
  TCachedName *iname = NULL;
	// Counter
  TCachedName *jname = NULL;	// Point to the tail
  TCachedName *lname = NULL;	// Current
  struct group *lostgrp = NULL;
  int retval = 0;
    
  iname = firstCachedGroup;
  jname = firstCachedGroup;
  lname = NULL;
  while ( iname != NULL ) {
    //printf("lookup: %s\n", iname->name );
    if ( 0 == strcmp(s, iname->name) ) break;
    jname = iname;
    iname = iname->next;
  }
  if (iname == NULL ) {
    //printf("scanning for %s\n", s );
    lname = (TCachedName*) calloc( 1, sizeof(TCachedName));
    lname->name = (char*) strdup(s);
    lname->uid = 0;
    lname->gid = 0;
    lname->next = NULL;
    if (firstCachedGroup==NULL) {
      firstCachedGroup = lname;
	// Add as head
    } else {
      jname->next = lname;
	// Append to the tail
    }
    // Now find the group in passwd
    lostgrp = getgrnam(s);
    lname->gid = lostgrp->gr_gid;  
    retval = lname->gid;
    //printf("- namecache: group %s == %i \n", s, retval );
  } else {
    retval = iname->gid;
    //printf("- namecache: group found %s at %s \n", s, iname->name );
  }
  return retval;
}

