/* Standard Includes */
#include <sys/stat.h>
#include <sys/sysmacros.h>

#include <stdio.h>
#include <string.h>
#include <unistd.h>

/* Local Includes */
#include "common_errors.h"
#include "blockcache.h"
#include "dirpath.h"
#include "gnutar.h"
#include "makedir.h"
#include "setup.h"
#include "setup_main.h"
#include "support.h"
#include "untar.h"

// Global vars
time_t      gstime;		// Time of the beggining of the installation

// Local vars
gzFile     *gzfd = NULL;	// Puntatore al file da decomprimere
int         gzheader = 1;	// Flag, 1 se bisogna leggere l'header; 0 blocco dati
int         gzcompleted = 0;	// Flag, 1 se estrazione completata
long        gzblock  = -1;	// Nr. di blocchi letti dal file compresso
TTarBlock   gzbuffer;		// buffer contenente il blocco dati attualmente letto
TCacheBlock gzentry;		// Dati sul file nel blocco corrente
char	   *loRootDir = NULL;   // lpopt->targetdir
int         loAction  = ACTION_UNKNOWN;  // lpopt->action
int	    loMode    = 0;        // untar(... mod)
int         loVerbose = 0;        // Livello di verbose
char        lo_source_file[256];  // Nome del file nel blocco corrente
time_t      lo_source_time;       // Data del file nel blocco corrente
long        lo_source_size = 0;   // Dimensione del file nel blocco corrente
char        lo_target_file[256];  // Nome del file target da creare
FILE       *lo_target_fd = NULL;  // Handle del file target da creare
long        lo_target_tocopy = 0; // Nr. di byte rimasti da copiare su target
int	    loFlagInstall = 0;	  // Flag 0=Normal, 1=trovato /install, 2=fine /install



/* 
 * Controlla l'operazione da effettuare in accordo con l'azione 
 * specificata. Se il file/directory passa il test viene creato
 * e la sua entry viene cacheata per essere in seguito elaborata
 * per applicare le giuste permissions.
 */
int process_file() {
  int     lo_target_exist;      // Flag, 0 se il target file gia' esiste
  struct  stat lo_stat;         // Struttura per controllare se un file esiste
  mode_t  lo_flag_block;
  int     lo_flag_makedir;
  int     lo_major, lo_minor;   // Major and minor of the device to create
  
  //printf( "%s (%li) ==> %s \n", lo_source_file, lo_source_size, lo_target_file );

  switch(loAction) {
      
    case ACTION_INSTALL:
    case ACTION_UPDATE:
    case ACTION_RESTORE:
    {

      // Controlla se il file target esiste e se bisogna installarlo o meno
      lo_target_exist = lstat( lo_target_file, &lo_stat );
      if ( lo_target_exist == 0 ) {
        //printf("%s esiste\n", lo_target_file );
        // Nel caso esiste, deve controllare la data ?
        if (loAction == ACTION_UPDATE) {
          if ( lo_stat.st_mtime == lo_source_time ) {
            if (loVerbose>0) printf("- file %s has the same age, skipping\n", lo_target_file);
            goto_next_header( gzfd, lo_source_size );
            return 0;
          } else {
            if (loVerbose>0) printf("- file %s has different age, updating\n", lo_target_file);
          }
        } else
        if (loAction == ACTION_RESTORE) {
          // File exist so skip it
          if (loVerbose>0) printf("- file %s exists skip it\n", lo_target_file);
          goto_next_header( gzfd, lo_source_size );
          return 0;
        }
      }

      // Il file e' da installare: procedi
      switch (gzbuffer.typeflag) {

        // File Normale
        case TAR_REGTYPE:
        case TAR_AREGTYPE: 
        {
          // Se il file esiste deve essere prima rimosso
          if (lo_target_exist == 0 ) unlink( lo_target_file );
          // Crea il file
          lo_target_fd = fopen( lo_target_file, "wb");
          if ( lo_target_fd == NULL) {
            /*
             * Impossibile creare il file, probabilmente non esiste
             * la directory in cui deve essere creato. Quindi proviamo
             * a creare la directory ed a riprovare la creazione del
             * file
             */
	    printf("untar: errore nella creazione del file %s, provo a creare la directory padre\n", lo_target_file); 
            makedir( 
              dirpath( lo_target_file ), 
              S_IRWXU | S_IRGRP | S_IXGRP | S_IROTH | S_IXOTH 
            );
            lo_target_fd = fopen(lo_target_file,"wb");
            if ( lo_target_fd == NULL) {
              lo_target_fd = NULL;
              printf("untar: impossibile creare il file %s\n", lo_target_file );
              //goto_next_header( gzfd, lo_source_size );
              return ERRORE_IO;
            }
          }
          /*
           * Il file e' stato creato, se la dimensione e' maggiore di
           * zero i blocchi successivi sono blocchi dati. Invice se
           * la dimensione e' 0 il prossimo blocco e' un'altra header
           * quindi dobbiamo creare un file di lunghezza 0
           */
          if (lo_source_size > 0 ) {
            lo_target_tocopy = lo_source_size;
            gzheader = 0;  // Get data Block
            if (loVerbose>0) printf("- creazione file normale %s\n", lo_target_file); 
          } else {
            fclose( lo_target_fd );
            lo_target_fd = NULL;
            gzheader = 1;
            if (loVerbose>0) printf("- creazione file a zero %s \n", lo_target_file);
          }
          push_block( &lo_target_file, &gzbuffer );
	  return 0;
	  // Fine file normale
        }

        // Hard Link
        case TAR_LNKTYPE: {
          if (loVerbose>0) printf("- hard link %s \n", lo_target_file );
          // Rimuove il file precedete se esiste
          if (lo_target_exist == 0 ) unlink( lo_target_file );
          /*
           * In caso di problemi, probabilmente la disposizione dei FS
           * e' diversa, quindi proviamo a creare un link simbolico
           * invece.
           */           
          if ( 0 != link( gzbuffer.linkname, lo_target_file ) ) {
            printf("untar: errore nella creazione hard-linking su %s, provo con un sym-link\n", gzbuffer.linkname );
            if ( 0 != symlink( gzbuffer.linkname, lo_target_file ) ) {
              printf("untar: impossibile creare anche il symlink\n" );
              return ERRORE_IO;
            }
          }
          push_block( &lo_target_file, &gzbuffer );
	  return 0;
	  // Fine Hard Link
        }

        // Lynk simbolico
        case TAR_SYMTYPE: {
          // Rimuove il precedente file se esiste
          if (lo_target_exist == 0 ) unlink( lo_target_file );
          if (loVerbose>0) printf("- creazione symlink su %s\n", gzbuffer.linkname );
          if ( 0 != symlink( gzbuffer.linkname, lo_target_file ) ) {
            printf("untar: errore creazione del symlink %s\n", lo_target_file );
            return ERRORE_IO;
          }
          push_block( &lo_target_file, &gzbuffer );
          return 0;
	  // fine symlink
        }

        // File di Device
        case TAR_CHRTYPE:
        case TAR_BLKTYPE: {
          // Remove first
          if (lo_target_exist == 0 ) unlink( lo_target_file );
          if (loVerbose>0) printf("- creazione del file di device %s\n", lo_target_file);
          if ( gzbuffer.typeflag == TAR_CHRTYPE ) {
            lo_flag_block = S_IFCHR;
          } else {
            lo_flag_block = S_IFBLK;
          }
	  lo_major = oct2int( gzbuffer.devmajor, 8);
	  lo_minor = oct2int( gzbuffer.devminor, 8);
          if ( 0 != mknod( lo_target_file, lo_flag_block, makedev( lo_major, lo_minor) ) ) {
            printf("untar: errore creazione device %s\n", lo_target_file );
            return ERRORE_IO;
	  }
          push_block( &lo_target_file, &gzbuffer );
          return 0;
	  // fine device
	}

        // Directory
        case TAR_DIRTYPE: {
          if (loVerbose>0) printf("- creazione directory %s\n", lo_target_file);
          if (lo_target_exist == 0 ) {
            //printf("	provo a rimuovere la directory\n");
            lo_flag_makedir = rmdir( lo_target_file );
          } else {
            lo_flag_makedir = 0;
          }
          if ( lo_flag_makedir == 0 ) {
            //printf("	creazione della directory 0700\n");
            if ( 0 != mkdir( lo_target_file,  S_IRWXU ) ) {
              printf("untar: errore nella creazione della directory %s", lo_target_file );
	      return ERRORE_IO;
            }
          } else {
            if (loVerbose>0) printf("- creazione dir %s non necessaria\n", lo_target_file);
          }
          push_block( &lo_target_file, &gzbuffer );
          return 0;
	  // fine dir
        }

        // Fifo             
        case TAR_FIFOTYPE: {
          // Remove first
          if (lo_target_exist == 0 ) unlink( lo_target_file );
          if (loVerbose>0) printf("- creazione della fifo %s\n", lo_target_file);
          if ( 0 == mkfifo( lo_target_file, S_IRWXU ) ) {
            push_block( &lo_target_file, &gzbuffer );
          } else {
            printf( "untar: errore durante la creazione della FIFO %s\n", lo_target_file);
	    return ERRORE_IO;
          }
          return 0;
	  // fine fifo
        }

        // File di tipo sconosciuto
        default: {
          printf( "untar: %s e' di tipo sconosciuto!\n", lo_source_file );
	  return ERRORE_PARAMETRO;
        }            
      // Fine switch tipo di file
      }
    // fine case su action
    }
    
    // default case for not handled yet actions
    default: {
      printf("untar: l'azione scelta (%i) non e' ancora supportata \n", loAction); 
      return ERRORE_PARAMETRO;
    }
  // fine switch su Action
  }
}



// Skip n block for 'size' bytes into the tar file
int goto_next_header(gzFile *fd, long size) {
  // Byte to skip to reach the other header
  z_off_t skip_byte;
  int     skip_block;
  if ( size > 0 ) {
    // Skip data Blocks
    skip_block = ( size / TAR_BLOCKSIZE );
    if ( size % TAR_BLOCKSIZE != 0 ) {
      skip_block++;
    }
    // Legge i blocchi dal file compresso
    skip_byte = skip_block * TAR_BLOCKSIZE;
    if ( -1 == gzseek( fd, skip_byte, SEEK_CUR ) ) {
      printf("untar: errore durante il posizionamento al blocco successivo a %li\n", gzblock );
      return ERRORE_IO;
    }
    // Avanza gzblock del nr. di blocchi da spostare
    gzblock += skip_block;
  }
  return 0;
}



/* 
 * Elabora il blocco, prima se e' un header, altrimenti se e' un blocco
 * dati viene aggiunto in coda all'ultimo file aperto
 */
 
int process_block() {

  // Elabora il blocco come header
  if ( gzheader == 1 ) {

    // Controlla che non sia un blocco terminatore    
    if (( gzbuffer.name[0] == 0 ) && ( gzbuffer.magic[0] == 0 )) {
      if (loVerbose>0) printf( "- estrazione archivio completata\n");
      gzcompleted=1;
      return 0;
    }

    // Controlla la corretta Intestazione del blocco
    if ( strncmp( gzbuffer.magic, TAR_MAGIC, 4 ) != 0 ) {
      printf( "untar: Block nr. %li is not an header Block!\n", gzblock );
      return ERRORE_IO;
    }


    /* 
     * Aggiunge al nome del file sorgente l'eventuale prefisso,
     * quindi compone il nome del file target aggiungendo la directory
     * d'installazione al nome del file.
     */
    // Se il file non ha prefisso usa il solo nome del file
    if ( gzbuffer.prefix[0] == 0 ) {
      strcpy( lo_source_file, gzbuffer.name );
    } else {
      // ... altrimenti mette il prefisso prima del nome
      strcpy( lo_source_file, gzbuffer.prefix);
      if ( lo_source_file[strlen(lo_source_file)] != '/' ) {
        strcat( lo_source_file, "/" );
      }
      strcat( lo_source_file, gzbuffer.name );
    }
    // Non contento aggiunge il nome della target directory
    strcpy( lo_target_file, loRootDir );
    strcat( lo_target_file, "/" );
    strcat( lo_target_file, lo_source_file );
    // Converte i valori di dimensione e data modifica
    lo_source_time = (time_t) oct2int( gzbuffer.mtime, 12 );
    lo_source_size = oct2int( gzbuffer.size, 12);

    // Controlla la maschera del file /install
    if ( loMode == UNTARMASK_INSTALL ) {
      if ( 0 != strncmp( "install/", lo_source_file, 8 ) ) {
        //printf("- Salto il file non in INSTALL %s\n", lo_source_file);
	if (loFlagInstall == 1 ) {
	  // Non ci sono piu' /install, termina pure la lettura dell'archivio
	  gzcompleted = 1;
	  return 0;
	}
        goto_next_header( gzfd, lo_source_size );	  
	return 0;
      }
      // Allora elabora il file, marchiamo che abbiamo trovato install
      loFlagInstall = 1;
      return process_file();    
    }

    // Controlla la maschera dei file nel caso incoming
    if ( 0 == strncmp( "incoming/", lo_source_file, 9 ) ) {
      //TODO: aggiustare INCOMING
      printf("Trovato INCOMING: %s\n", lo_source_file );
    }
    return process_file();    
      
  // Elabora il blocco come dati
  } else {

    // Controlla se c'e' un handle del file aperto
    if ( lo_target_fd != NULL ) {
      // Controlla se e' l'ultimo blocco del file
      if ( lo_target_tocopy > TAR_BLOCKSIZE ) {
        // Aggiunge blocco dati in coda al file
        //printf("untar: aggiunta blocco dati\n");
        if ( fwrite( &gzbuffer, sizeof(char), TAR_BLOCKSIZE, lo_target_fd ) != TAR_BLOCKSIZE ) {
          // In caso di errore non lascia il file monco sul FS
          printf( "untar: error writing %s skipping...\n", lo_target_file );
          fclose(lo_target_fd);
          unlink(lo_target_file);
          lo_target_fd = NULL;
          gzheader = 1;
        }
        // Diminuisce in numero di byte da copiare
        lo_target_tocopy -= TAR_BLOCKSIZE;
        //printf("untar: rimangono byte %li\n", lo_target_tocopy );
      } else {
        //printf("untar: aggiunta ultimo blocco dati\n");
        if (lo_target_tocopy > 0 ) {
          if ( fwrite( &gzbuffer, sizeof(char), lo_target_tocopy, lo_target_fd ) != lo_target_tocopy ) {
            printf("untar: error writing last Block of %s skipping...\n", lo_target_file );
            fclose(lo_target_fd);
            unlink(lo_target_file);
            lo_target_fd = NULL;
            gzheader = 1;
          }
        }
        fclose( lo_target_fd );
        //file_settime( lo_target_file, lo_source_time );
        lo_target_fd = NULL;
        gzheader = 1;
        //printf("- scrittura di %s completata\n", lo_target_file);
      }
    } else {
      printf("untar: blocco dati nr. %li ma nessun descrittore di file apero !\n", gzblock );
      return ERRORE_IO; 
    }
  }
  return 0;
}


/*
  STRATEGY
  To avoid problem with file and directory permissions we first extract
  all the files with mode 0700 and uid/gid root.root, then every entry
  is cached. At the end of the extraction process we apply all the
  cached entries.
*/

int untar(TSetupOptions *lpopt, int mode) {
  int     gzerror = 0;		// Valore di ritorno per questa fuznione
  int     gzbyte  = -1;		// Nr. di byte letti dal file compresso

  // Assign local vars
  loRootDir = lpopt->targetdir;
  loAction  = lpopt->action;
  loVerbose = lpopt->verbose;
  loMode    = mode;
  gstime    = time(NULL);
  
  // Open .tar.gz file
  gzfd = gzopen( lpopt->sourcefile, "rb" );
  if (gzfd == NULL) {
    printf( "untar: gzopen, impossibile aprire il file compresso %s\n", lpopt->sourcefile );
    return ERRORE_IO;
  }

  // Set mask to 0700
  umask( S_IRWXG | S_IRWXO );

  // Read Data Blocks
  gzblock = -1;
  gzheader = 1;
  gzerror = 0;
  gzcompleted = 0;
  while ((gzerror == 0) && (gzcompleted == 0)) {

    // Legge un blocco di dati non compressi
    gzblock++;
    gzbyte = gzread( gzfd, (char*)(&gzbuffer), TAR_BLOCKSIZE );

    // Controlla la corretta decompressione dei dati
    if ( gzbyte < 0 ) {
      printf( "untar: errore (%i) nella zlib al blocco %li\n", gzbyte, gzblock );
      break;
    } else
    if ( gzbyte == 0 ) {
      if (loVerbose>0) printf( "- decompressione archivio completata, %li blocchi.\n", gzblock);
      break;
    } else
    if ( gzbyte != TAR_BLOCKSIZE ) {
      printf( "untar: letti solo %i byte dal blocco %li\n", gzbyte, gzblock );
      gzerror = ERRORE_IO;
      break;
    }

    // Elabora il blocco letto
    gzerror = process_block();
  }
  
  // END untar
  if ( Z_OK != gzclose( gzfd ) ) {
    printf( "untar: fallito gzclose()\n");
    return ERRORE_IO;
  }

  // FlushCache
  if (gzerror == 0) {
    gzerror = FlushCache();
  }
  
  // Exit
  return gzerror;
}
